﻿using Microsoft.Extensions.Logging;
using SushiBarBusinessLogic.MailWorker;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Enums;

namespace SushiBarBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly AbstractMailWorker _mailWorker;
        private readonly IClientLogic _clientLogic;
        static readonly object locker = new object();
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage,
            AbstractMailWorker mailWorker, IClientLogic clientLogic)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _mailWorker = mailWorker;
            _clientLogic = clientLogic;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() :
                _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            _logger.LogInformation("ReadElement.");
            var element = model == null ? null : _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement return null list");
                return null;
            }
            _logger.LogInformation("ReadElement Id:{ Id}", element.Id);
            return element;
        }
        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Wrong Order Status");
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            model.Status = OrderStatus.Принят;
            var res = _orderStorage.Insert(model);
            if (res == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            SendOrderStatusMail(model.ClientId, $"Изменен статус заказа #{res.Id}", $"Заказ #{res.Id} изменен статус на {model.Status}");
            return true;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return ToNextStatus(model, OrderStatus.Выполняется);
        }
        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ToNextStatus(model, OrderStatus.Выдан);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ToNextStatus(model, OrderStatus.Готов);
        }

        public bool ToNextStatus(OrderBindingModel model, OrderStatus orderStatus)
        {
            CheckModel(model, false);
            var element = _orderStorage.GetElement(new OrderSearchModel()
            {
                Id = model.Id
            });
            if (element == null)
            {
                throw new ArgumentNullException(nameof(element));
            }

            if (element.ImplementerId != null && model.ImplementerId != null && model.ImplementerId != element.ImplementerId)
            {
                throw new InvalidOperationException(nameof(model.ImplementerId));
            }

            model.SushiId = element.SushiId;
            model.DateCreate = element.DateCreate;
            model.DateImplement = element.DateImplement;
            model.Status = element.Status;
            model.Count = element.Count;
            model.Sum = element.Sum;
            model.ClientId = element.ClientId;
            model.ImplementerId = element.ImplementerId.HasValue ? element.ImplementerId : model.ImplementerId;

            if (model.Status != orderStatus - 1)
            {
                _logger.LogWarning("Status update to " + orderStatus + " operation failed");
                return false;
            }
            model.Status = orderStatus;

            if (model.Status == OrderStatus.Выдан)
            {
                model.DateImplement = DateTime.Now;
            }

            var result = _orderStorage.Update(model);
            if (result == null)
            {
                model.Status--;
                _logger.LogWarning("Changing status operation faled");
                return false;
            }
            SendOrderStatusMail(result.ClientId, $"Изменен статус заказа #{result.Id}", $"Заказ #{model.Id} изменен статус на {result.Status}");
            return true;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.DateImplement != null && model.DateCreate > model.DateImplement)
            {
                throw new InvalidOperationException("Заказ не может быть начат после его завершения");
            }
            if (model.Count <= 0)
            {
                throw new ArgumentException("Количество продукта должно быть бльше 0");
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
        }
        private bool SendOrderStatusMail(int clientId, string subject, string text)
        {
            try
            {
                var client = _clientLogic.ReadElement(new() { Id = clientId });
                if (client == null)
                {
                    return false;
                }
                _mailWorker.MailSendAsync(new()
                {
                    MailAddress = client.Email,
                    Subject = subject,
                    Text = text
                });
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
            
        }
    }
}
